@@ -19,7 +19,11 @@ Note: Groups are room-dependent and case-insensitive.
 import logging
 import re
 
+from nio import RoomMessage, MatrixRoom
+
 from chaanbot import command_utility
+from chaanbot.database import Database
+from chaanbot.matrix import Matrix
 
 logger = logging.getLogger("highlight")
 
@@ -45,7 +49,7 @@ class Highlight:
         },
     }
 
-    def __init__(self, config, matrix, database, requests):
+    def __init__(self, config, matrix: Matrix, database: Database, requests):
         self.matrix = matrix
         if database:
             self.database = database
@@ -62,20 +66,20 @@ class Highlight:
         else:
             logger.info("No database provided, highlight module disabled")
 
-    def run(self, room, event, message) -> bool:
+    async def run(self, room: MatrixRoom, event: RoomMessage, message) -> bool:
         if self._should_run(message):
             if command_utility.matches(self.operations["highlight_all"], message):
                 logger.debug("Highlighting all")
-                self._highlight_all(room, event["sender"], message)
+                await self._highlight_all(room, event.sender, message)
             elif command_utility.matches(self.operations["highlight"], message):
                 logger.debug("Highlighting")
-                self._highlight(room, event["sender"], message)
+                await self._highlight(room, event.sender, message)
             elif command_utility.matches(self.operations["add_to_group"], message):
                 logger.debug("Adding to group")
-                self._add_or_create_group(room, message)
+                await self._add_or_create_group(room, message)
             elif command_utility.matches(self.operations["delete_from_group"], message):
                 logger.debug("Deleting from group")
-                self._delete_from_group(room, message)
+                await self._delete_from_group(room, message)
             else:
                 raise RuntimeError("Could not find command to run on message, but should have been able to")
             return True
@@ -84,26 +88,26 @@ class Highlight:
     def _should_run(self, message) -> bool:
         return self.database and command_utility.matches(self.operations, message)
 
-    def _highlight_all(self, room, sender_user_id, message):
-        user_ids = [user.user_id for user in room.get_joined_members()]
+    async def _highlight_all(self, room: MatrixRoom, sender_user_id, message):
+        user_ids = [user.user_id for user in room.users.values()]
         argument = command_utility.get_argument(message)
         user_ids = self._remove_element(user_ids, sender_user_id)
         if user_ids:
             message = ", ".join(user_ids)
             logger.debug("Highlighting: {}".format(message))
             if argument:
-                room.send_text(message + ": " + argument)
+                await self.matrix.send_text_to_room(message + ": " + argument, room.room_id)
             else:
-                room.send_text(message)
+                await self.matrix.send_text_to_room(message, room.room_id)
         else:
             logger.debug("No users to highlight in room {}".format(room.room_id))
-            room.send_text("No users to highlight")
+            await self.matrix.send_text_to_room("No users to highlight", room.room_id)
         return
 
-    def _highlight(self, room, sender_user_id, message):
+    async def _highlight(self, room: MatrixRoom, sender_user_id, message):
         argument = command_utility.get_argument(message)
         if not argument:
-            room.send_text("Correct syntax is !hl [group] [optional text].")
+            await self.matrix.send_text_to_room("Correct syntax is !hl [group] [optional text].", room.room_id)
             return
         arguments = argument.split(None, 1)
         group = arguments[0].lower()
@@ -113,21 +117,23 @@ class Highlight:
 
             if len(arguments) > 1:
                 argument = arguments[1]
-                room.send_text(members + ": " + argument)
+                await self.matrix.send_text_to_room(members + ": " + argument, room.room_id)
             else:
-                room.send_text(members)
+                await self.matrix.send_text_to_room(members, room.room_id)
         else:
-            room.send_text("Group \"{}\" does not have any members to highlight".format(group))
+            await self.matrix.send_text_to_room("Group \"{}\" does not have any members to highlight".format(group),
+                                                room.room_id)
 
-    def _add_or_create_group(self, room, message):
+    async def _add_or_create_group(self, room: MatrixRoom, message):
         arguments = command_utility.get_argument(message).split()
         group = arguments[0].lower()
         users_to_add = arguments[1:]
         logger.debug("User wants to add {} to {}".format(users_to_add, group))
         if group and len(users_to_add) > 0:
             for user in users_to_add:
-                if not self.matrix.get_user(room, user):
-                    room.send_text("User: \"{}\" is not in room".format(user))
+                found_user = self.matrix.get_user(room, user)
+                if not found_user:
+                    await self.matrix.send_text_to_room("User: \"{}\" is not in room".format(user), room.room_id)
                     return
 
             new_members = []
@@ -147,14 +153,18 @@ class Highlight:
                         new_members.append(user)
 
             if new_members:
-                room.send_text("Added \"{}\" to group \"{}\"".format(", ".join(new_members), group))
+                await self.matrix.send_text_to_room(
+                    "Added \"{}\" to group \"{}\"".format(", ".join(new_members), group),
+                    room.room_id)
             else:
-                room.send_text("Could not add \"{}\" to group \"{}\"".format(", ".join(users_to_add), group))
+                await self.matrix.send_text_to_room(
+                    "Could not add \"{}\" to group \"{}\"".format(", ".join(users_to_add), group), room.room_id)
         else:
-            room.send_text("Could not add: \"{}\" to group \"{}\". Syntax is incorrect".format(group, users_to_add))
+            await self.matrix.send_text_to_room(
+                "Could not add: \"{}\" to group \"{}\". Syntax is incorrect".format(group, users_to_add), room.room_id)
         return
 
-    def _delete_from_group(self, room, message):
+    async def _delete_from_group(self, room: MatrixRoom, message):
         arguments = command_utility.get_argument(message).split()
         group = arguments[0].lower()
         members_to_remove = arguments[1:]
@@ -162,7 +172,7 @@ class Highlight:
         if group and len(members_to_remove) > 0:
             for member in members_to_remove:
                 if not self.matrix.get_user(room, member):
-                    room.send_text("User: {} is not in room".format(member))
+                    await self.matrix.send_text_to_room("User: {} is not in room".format(member), room.room_id)
                     return
 
             removed_members = []
@@ -178,15 +188,19 @@ class Highlight:
             if removed_members:
                 logger.debug(
                     "Removed {} from group {} in room {}".format(", ".join(removed_members), group, room.room_id))
-                room.send_text("Removed \"{}\" from group \"{}\"".format(", ".join(removed_members), group))
+                await self.matrix.send_text_to_room(
+                    "Removed \"{}\" from group \"{}\"".format(", ".join(removed_members), group), room.room_id)
             else:
                 logger.debug(
                     "Could not remove {} from group {} in room {}".format(", ".join(members_to_remove), group,
                                                                           room.room_id))
-                room.send_text("Could not remove \"{}\" from group \"{}\"".format(", ".join(members_to_remove), group))
+                await self.matrix.send_text_to_room(
+                    "Could not remove \"{}\" from group \"{}\"".format(", ".join(members_to_remove), group),
+                    room.room_id)
         else:
-            room.send_text(
-                "Could not remove: \"{}\" from group \"{}\". Syntax is incorrect".format(group, members_to_remove))
+            await self.matrix.send_text_to_room(
+                "Could not remove: \"{}\" from group \"{}\". Syntax is incorrect".format(group, members_to_remove),
+                room.room_id)
         return
 
     @staticmethod
@@ -197,13 +211,13 @@ class Highlight:
         result = result.fetchone()
         return result is not None
 
-    def _get_members(self, room, group) -> list:
+    def _get_members(self, room: MatrixRoom, group) -> list:
         return [row[0] for row in
                 self.database.connect().execute(
                     "SELECT member FROM highlight_groups WHERE room_id = ? AND group_name = ?",
                     (room.room_id, group)).fetchall()]
 
-    def _get_member_user_ids_except_sender(self, room, group, sender_user_id) -> list:
+    def _get_member_user_ids_except_sender(self, room: MatrixRoom, group, sender_user_id) -> list:
         member_user_ids = [self.matrix.get_user(room, member).user_id for member in self._get_members(room, group)]
         return self._remove_element(member_user_ids, sender_user_id)
 
